#include "Image.h"

int Image::nextID;

Image::Image(char *filename) {
   FILE *file;
   unsigned long size;                 /*  size of the image in bytes. */
   unsigned long i;                    /*  standard counter. */
   unsigned short int planes;          /*  number of planes in image (must be 1)  */
   unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
   char temp;                          /*  used to convert bgr to rgb color. */

   mID = nextID;
   nextID++;

   /*  make sure the file is there. */
   if ((file = fopen(filename, "rb"))==NULL) {
      printf("File Not Found : %s\n",filename);
   } else {
      /*  seek through the bmp header, up to the width height: */
      fseek(file, 18, SEEK_CUR);

      /*  No 100% errorchecking anymore!!! */

      /*  read the width */
      mSizeX = GetInt (file);

      /*  read the height */ 
      mSizeY = GetInt (file);

      /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
      size = mSizeX * mSizeY * 3;

      /*  read the planes */    
      planes = GetShort(file);
      if (planes != 1) {
         printf("Planes from %s is not 1: %u\n", filename, planes);
      } else {
         /*  read the bpp */    
         bpp = GetShort(file);
         if (bpp != 24) {
            printf("Bpp from %s is not 24: %u\n", filename, bpp);
         } else {
            /*  seek past the rest of the bitmap header. */
            fseek(file, 24, SEEK_CUR);
               
            /*  read the data.  */
            mData = (char *) malloc(size);
            if (mData == NULL) {
               printf("Error allocating memory for color-corrected image data");
            } else {
               if ((i = fread(mData, size, 1, file)) != 1) {
                  printf("Error reading image data from %s.\n", filename);
               } else {
                  for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
                     temp = mData[i];
                     mData[i] = mData[i+2];
                     mData[i+2] = temp;
                  }
                     
                  fclose(file); /* Close the file and release the filedes */
               }
            }
         }
      }
   }

   glBindTexture(GL_TEXTURE_2D, mID);
   glTexImage2D(GL_TEXTURE_2D, 0, 3,
      mSizeX, mSizeY,
      0, GL_RGB, GL_UNSIGNED_BYTE, mData);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
}

unsigned int Image::GetInt(FILE *fp) {
  int c, c1, c2, c3;
  
  /*  get 4 bytes */ 
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

unsigned int Image::GetShort(FILE *fp){
  int c, c1;
  
  /* get 2 bytes*/
  c = getc(fp);  
  c1 = getc(fp);
  
  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

