#include "Model.h"
#include <fstream>
#include <assert.h>

using namespace std;

bool Face::contains(int v) {
   return (v1 == v || v2 == v || v3 == v);
}

Model::Model(char *filename){
  //initialization
   max_x = max_y = max_z = FLT_MIN;
   min_x = min_y = min_z = FLT_MAX;
   max_extent = 1.0;
  
  //make sure a file to read is specified
   ReadFile(filename);
   for (size_t index = 0; index < mNormals.size(); index++) {
      mNormals[index]->Norm();
   }
    //once the file is parsed find out the maximum extent to center and scale mesh
   max_extent = max_x - min_x;
   if (max_y - min_y > max_extent) max_extent = max_y - min_y;
   max_extent *= PIECE_SCALE;
   mCenter.x = mCenter.x/mVertices.size();
   mCenter.y = mCenter.y/mVertices.size();
   mCenter.z = mCenter.z/mVertices.size();

   mEdges = new vector<vector<Edge> >(mVertices.size());
}

Model *Model::Clone()
{
   vector<Face *>::iterator fitr;
   vector<MyVector *>::iterator vitr;
   Model *model = new Model();
   Face *face;
   int vNum = 1;

   for (fitr = mFaces.begin(); fitr != mFaces.end(); fitr++) {
      face = new Face(**fitr);
      model->mFaces.push_back(face);
      model->mVertices.push_back(new MyVector(*mVertices[(*fitr)->v1 - 1]));
      model->mNormals.push_back(new MyVector(*mNormals[(*fitr)->v1 - 1]));
      face->v1 = vNum;
      vNum++;
      model->mVertices.push_back(new MyVector(*mVertices[(*fitr)->v2 - 1]));
      model->mNormals.push_back(new MyVector(*mNormals[(*fitr)->v2 - 1]));
      face->v2 = vNum;
      vNum++;
      model->mVertices.push_back(new MyVector(*mVertices[(*fitr)->v3 - 1]));
      model->mNormals.push_back(new MyVector(*mNormals[(*fitr)->v3 - 1]));
      face->v3 = vNum;
      vNum++;
   }

   for (vitr = mNormals.begin(); vitr != mNormals.end(); vitr++) {
      model->mNormals.push_back(new MyVector(**vitr));
   }

   model->mCenter = mCenter;

   model->max_x = max_x;
   model->max_y = max_y;
   model->max_z = max_z;
   model->min_x = min_x;
   model->min_y = min_y;
   model->min_z = min_z;
   model->max_extent = max_extent;

   return model;
}

//process each line of input save vertices and faces appropriately
void Model::ReadLine(char* str) {
  
  int indx = 0, vi;
  float x = 0, y = 0, z = 0;
  float r, g, b;
  int mat;
  MyVector v1, v2;
  
  if (str[0]=='#') return;
  //read a vertex or face
  if (str[0]=='V' && !strncmp(str,"Vertex ",7)) {
    MyVector *v, *n;
    assert(sscanf(str,"Vertex %d %g %g %g",&vi,&x,&y,&z)==4);
    v = new MyVector(x, y, z);
    n = new MyVector(0,0,0);
    //store the vertex
    mVertices.push_back(v);
    mNormals.push_back(n);
    //house keeping to display in center of the scene
    mCenter.x += v->x;
    mCenter.y += v->y;
    mCenter.z += v->z;
    if (v->x > max_x) max_x = v->x; if (v->x < min_x) min_x = v->x;
    if (v->y > max_y) max_y = v->y; if (v->y < min_y) min_y = v->y;
    if (v->z > max_z) max_z = v->z; if (v->z < min_z) min_z = v->z;
  } else if (str[0]=='F' && !strncmp(str,"Face ",5)) {
    Face* t;
    t = new Face();
    char* s=str+4;
    int fi=-1;
    for (int t_i = 0;;t_i++) {
      while (*s && isspace(*s)) s++;
      //if we reach the end of the line break out of the loop
      if (!*s) break;
      //save the position of the current character
      char* beg=s;
      //advance to next space
      while (*s && isdigit(*s)) s++;
      //covert the character to an integer
      int j=atoi(beg);
      //the first number we encounter will be the face index, don't store it
      if (fi<0) { fi=j; continue; }
      //otherwise process the digit we've grabbed in j as a vertex index
      //the first number will be the face id the following are vertex ids
      if (t_i == 1)
        t->v1 = j;
      else if (t_i == 2)
        t->v2 = j;
      else if (t_i == 3)
        t->v3 = j;
      //if there is more data to process break out
      if (*s =='{') break;
    }
    //possibly process colors if the mesh has colors
    if (*s && *s =='{'){
      char *s1 = s+1;
      std::cout << "trying to parse color " << !strncmp(s1,"rgb",3) << std::endl;
      //if we're reading off a color
      if (!strncmp(s1,"rgb=",4)) {
        //grab the values of the string
        assert(sscanf(s1,"rgb=(%g %g %g) matid=%d",&r,&g,&b,&mat)==4);
      }
    }
    v1 = mVertices[t->v1 - 1]->Minus(*mVertices[t->v2 - 1]);
    v2 = mVertices[t->v1 - 1]->Minus(*mVertices[t->v3 - 1]);
    t->mNormal = v1.CrossProd(v2);
    t->mNormal.Norm();

    mNormals[t->v1 - 1]->x += t->mNormal.x;
    mNormals[t->v1 - 1]->y += t->mNormal.y;
    mNormals[t->v1 - 1]->z += t->mNormal.z;

    mNormals[t->v2 - 1]->x += t->mNormal.x;
    mNormals[t->v2 - 1]->y += t->mNormal.y;
    mNormals[t->v2 - 1]->z += t->mNormal.z;

    mNormals[t->v3 - 1]->x += t->mNormal.x;
    mNormals[t->v3 - 1]->y += t->mNormal.y;
    mNormals[t->v3 - 1]->z += t->mNormal.z;
    
    //store the triangle read in
    mFaces.push_back(t);
  }
}

//process the input stream from the file
void Model::ReadStream(std::istream& is)
{
  char str[256];
  for (;is;) {
     is >> std::ws;
    is.get(str,sizeof(str));
    if (!is) break;
    is.ignore(9999,'\n');
    ReadLine(str);
  }
}

//open the file for reading
void Model::ReadFile(char* filename) {
  
  printf("Reading coordinates from %s\n", filename);
  
  ifstream in_f(filename);
  if (!in_f)  {
    printf("Could not open file %s\n", filename);
  } else {
    ReadStream(in_f);
  }
}
